 // Package 
 

 
 package my_agent_pkg;
   
    `include "uvm_macros.svh"
   //Agent Includes- Transaction, Sequence, Sequencer, Driver, Agent 
   
   import uvm_pkg::*;
   
   //////////////////////////
   //Transaction Class
   
   class my_transaction extends uvm_sequence_item;
  
  `uvm_object_utils(my_transaction)
  
     rand bit d;
     bit q;
     
     constraint c1 {d>=0;d<=1;}
     

     function new(string name="my_transaction");
       super.new(name);
       
    endfunction:new
    
    function string convert2string();
      return $sformatf("%sd:\t%b\tq:%b",super.convert2string(),d,q);
      
    endfunction:convert2string
    
  endclass:my_transaction 
  
  //////////////////////////
  //END of Transaction Class
  

  //////////////////////////
  //Sequencer Class
  
  class my_sequencer extends uvm_sequencer #(my_transaction);
    `uvm_component_utils(my_sequencer)
    
    function new(string name="my_sequencer",uvm_component parent=null);
      super.new(name,parent);
    endfunction:new
    
  endclass:my_sequencer 
 //////////////////////////
 // END of Sequencer Class 
 
 
 //////////////////////////
 // Sequence Class. NOT an UVM component. Extends Transaction 
 
 class my_sequence extends uvm_sequence#(my_transaction);
   
   `uvm_object_utils(my_sequence)
   
   function new(string name="my_sequence");
     super.new(name);
   endfunction:new
   
   // Task Body. Behavior of the sequence. 
   task body;
     repeat(10) 
     begin
       my_transaction tx;
       tx=my_transaction::type_id::create("tx");
       
       start_item(tx);
       
       assert(tx.randomize());
       finish_item(tx);
      
     end
   endtask:body 
   
 endclass:my_sequence 

//////////////////////////
//END of My Sequence Class 

//////////////////////////
//Driver Class

class my_driver extends uvm_driver #(my_transaction);
  
  `uvm_component_utils(my_driver)
  
  virtual dflop_if dflop_vi;
  

  
  
        function new(string name="my_driver",uvm_component parent=null);
          super.new(name,parent);
        endfunction:new 
   
   function void build_phase(uvm_phase phase);
       super.build_phase(phase);

      
      if(!uvm_config_db#(virtual dflop_if)::get(this, "", "dflop_vi", dflop_vi))
       `uvm_fatal("NOVIF",{"virtual interface must be set for: ",get_full_name(),".vif"});
      
              endfunction:build_phase
        
  task run_phase(uvm_phase phase);
           
          phase.raise_objection(this);
          repeat(10)
          begin
            my_transaction tx;
            @(posedge dflop_vi.clk)
          seq_item_port.get_next_item(tx);
          
          dflop_vi.d= tx.d;
          dflop_vi.q= tx.q;
   
       
          
          
          seq_item_port.item_done();
             end
             
             @(posedge dflop_vi.clk)
        phase.drop_objection(this);
        
      endtask:run_phase  
      

endclass:my_driver
 
//////////////////////////
//END of Driver Class


//////////////////////////
//Monitor Class

class my_monitor extends uvm_monitor;
  
  `uvm_component_utils(my_monitor)
  
  virtual dflop_if dflop_vi;
  
 function new(string name="my_monitor",uvm_component parent=null);
      super.new(name,parent);
    endfunction:new
    
  function void build_phase(uvm_phase phase);
  super.build_phase(phase);
  
  if(!uvm_config_db#(virtual dflop_if)::get(this, "", "dflop_vi", dflop_vi))
       `uvm_fatal("NOVIF",{"virtual interface must be set for: ",get_full_name(),".vif"});
       
     endfunction:build_phase 
     
     task run_phase(uvm_phase phase);
       repeat(10)
       begin 
         my_transaction tx;
        
         @(posedge dflop_vi.clk)
         begin
            tx=my_transaction::type_id::create("tx");        
      
         tx.d=dflop_vi.d;
 
         tx.q=dflop_vi.q;
         
         
          uvm_report_info("my_monitor",tx.convert2string());
        end
        end
        
      endtask
    endclass:my_monitor 
         
  
    

//////////////////////////
//Agent Class

class my_agent extends uvm_agent;
  `uvm_component_utils(my_agent)
  
  my_sequencer my_sequencer_h;
  my_driver my_driver_h;
  my_monitor my_monitor_h;
  
  virtual dflop_if dflop_vi; // Handle for the interface
  
   function new(string name="my_agent",uvm_component parent=null);
      super.new(name,parent);
    endfunction:new
    
    function void build_phase(uvm_phase phase);
     // super.build_phase(phase);
      
      //propagate interface using config db
      
      if(!uvm_config_db#(virtual dflop_if)::get(this, "", "dflop_vi", dflop_vi))
       `uvm_fatal("NOVIF",{"virtual interface must be set for: ",get_full_name(),".vif"});

      my_sequencer_h=my_sequencer::type_id::create("my_sequencer_h",this);
      
      my_driver_h=my_driver::type_id::create("my_driver_h",this);
      
      my_monitor_h=my_monitor::type_id::create("my_monitor_h",this);
      
      
    endfunction:build_phase
    
    function void connect_phase(uvm_phase phase);
      
      my_driver_h.seq_item_port.connect(my_sequencer_h.seq_item_export);
    endfunction:connect_phase 

endclass:my_agent 

//////////////////////////
//END of Agent Class


endpackage:my_agent_pkg 
   
